'''
A unified interface to builtin hashers and codecs as well as some of my own.

AUTHORS:
v0.2.0+             --> pydsigner
'''

import hashlib
import base64
import zlib
import binascii
import random
import math
import time
import decimal
from decimal import Decimal as dec
import string
try:
    import urllib.parse as urler
except ImportError:
    import urllib as urler

from . import iter_utils
from .compatibility import str, range, chr
from .math_utils import sane_hex


__all__ = ['rand_key', 'multi_pass', 'encoder_classes', 'fetcher', 'Triplets', 
           'PD2', 'PD3', 'PD4', 'PD5', 'PD6', 'PD7', 'PD8', 'PD9', 'PD10', 
           'PD11', 'PD12', 'PD13', 'PD14', 'RD', 'SHA1', 'SHA224', 'SHA256', 
           'SHA384', 'SHA512', 'MD5', 'Base16', 'Base32', 'Base64', 'Hex', 
           'HQX', 'UU', 'URL', 'HTML', 'CRC32', 'Adler32']


class Triplets(object):
    '''
    An encoder useful mostly for minor ciphers or sending information in a 
    format using a small amount of different characters.
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return '.'.join([str(ord(c) * 3) for c in s])
    def decode(self, s):
        return ''.join([chr(int(c) // 3) for c in s.split('.')])


class PD2(object):
    '''
    A basic hasher based on PD1(), but with many improvements. This is only 
    recommended for checksums: for passwords, PD6 or a more common hasher 
    should be used.
    
    AUTHORS:
    v0.2.4+             --> pydsigner
    '''
    mag_num = dec('8.33')
    max_len = 30
    
    def encode(self, s):
        L = int(len(s) * self.mag_num)
        res = [str(ord(c) * L) for c in s]
        return ''.join(res)[:self.max_len]


class PD3(object):
    '''
    A more advanced hasher based on PD2(). Should be suitable for passwords, 
    as well as checksums, especially as it is a new hasher. NOTE: This is slow!
    
    AUTHORS:
    v0.2.6+             --> pydsigner
    '''
    decimal.getcontext().prec = 15
    mag_num = dec(5 ** 2 + 9 ** 2).sqrt().rotate(6)
    max_len = 64
    
    def encode(self, s):
        L = int(dec(len(s)) * self.mag_num)
        res = [str(ord(c) * L) for c in s]
        # how slow will this make this hasher?
        decimal.getcontext().prec = 77
        d = dec(''.join(res)).rotate(-1)
        return sane_hex(d)[:self.max_len]


class PD4(object):
    '''
    A more advanced hasher based on PD3(). Should be suitable for passwords, 
    but you may wish to look at PD5(), PD7(), or PD8() for checksums. 
    NOTE: This is VERY slow!
    
    AUTHORS:
    v0.2.9+             --> pydsigner
    '''
    decimal.getcontext().prec = 15
    mag_num = dec(6 ** 2 + 1).sqrt().rotate(-9)
    max_len = 96
    
    def encode(self, s):
        L = int(dec(len(s)) * self.mag_num)
        res = ''.join([str(hash(c) * L) for c in s]).replace('-', '56')
        # Is speed going to be a problem here?
        decimal.getcontext().prec = 116
        d = dec(res).rotate(-2)
        return sane_hex(d)[:self.max_len]


class PD5(object):
    '''
    A hasher that is much faster than PD4(), but more suited for generating 
    checksums than password storage because of its limited length (23 - 30 
    characters)
    
    AUTHORS:
    v0.3.0+             --> pydsigner
    '''
    max_len = 128
    mag_num = int(dec(40).exp())
    
    def encode(self, s):
        L = self.mag_num * len(s)
        r = int(str(hash(s) * L).replace('-', str(self.max_len)))
        return sane_hex(r)[:self.max_len]


class PD6(object):
    '''
    A hasher that is slower than PD5(), but better suited for password 
    storage and much faster than PD4(). WARNING: It is not recommended that one 
    use this hasher on strings more than 256 characters long.
    
    AUTHORS:
    v0.3.1+             --> pydsigner
    '''
    max_len = 128
    mag_num = int(dec(46).exp())
    split_size = 2
    dash = mag_num // 3
    
    def encode(self,  s):
        z = len(s) // self.split_size
        z += 1 if len(s) % self.split_size else 0
        bitesize = self.max_len * 2 * self.split_size
        sec = iter_utils.section(s[:bitesize], self.split_size)
        L = self.mag_num * z
        r = int(''.join([str(hash(s) * L).replace('-', str(self.dash)
                ) for s in sec]))
        return sane_hex(r)[:self.max_len]


class PD7(PD6):
    '''
    A fast checksum generator similar to, but slower than, the popular md5. 
    For a longer generator, see PD8(). WARNING: It is not recommended that one 
    use this hasher on strings more than 128 characters long.
    
    AUTHORS:
    v0.3.6+             --> pydsigner
    '''
    max_len = 32
    mag_num = 68
    split_size = 4
    dash = mag_num // 5


class PD8(PD6):
    '''
    A checksum generator designed to be right between PD7() and PD6() in 
    terms of speed and quality. WARNING: It is not recommended that one use 
    this hasher on strings more than 192 characters long.
    
    AUTHORS:
    v0.3.4+             --> pydsigner
    '''
    max_len = 64
    mag_num = 57
    split_size = 3
    dash = (mag_num * 6) // 19


class PD9(object):
    '''
    A hasher designed to replace PD6().
    
    AUTHORS:
    v0.3.5+             --> pydsigner
    '''
    max_len = 128
    mag_num = int(dec(53).exp())
    split_size = 2
    dash = str((mag_num * 2) // 7)
    
    def encode(self, s):
        z = len(s) // self.split_size
        z += 1 if len(s) % self.split_size else 0
        bitesize = self.max_len * 2 * self.split_size
        sec = iter_utils.section(s[:bitesize], self.split_size)
        L = self.mag_num * z
        v = ''.join([str(hash(s[bitesize:]))] + 
                    [str(hash(s) * L).replace('-', self.dash) for s in sec])
        r = int(v)
        return sane_hex(r)[:self.max_len]


class PD10(PD9):
    '''
    A hasher designed to replace PD7().
    
    AUTHORS:
    v0.3.5+             --> pydsigner
    '''
    max_len = 32
    mag_num = int(dec(57).exp())
    split_size = 4
    dash = str(mag_num // 4)


class PD11(PD9):
    '''
    A hasher designed to replace PD8().
    
    AUTHORS:
    v0.3.5+             --> pydsigner
    '''
    max_len = 64
    mag_num = int(dec(43).exp()) * 9
    split_size = 3
    dash = str(int(dec(31).exp()))


class PD12(object):
    '''
    Similar to but not the same as PD9().
    
    AUTHORS:
    v0.3.6+             --> pydsigner
    '''
    max_len = 128
    mag_num = 7
    split_size = 2
    dash = str(int(dec(23).exp()))
    
    def encode(self, s):
        z = len(s) // self.split_size
        z += 1 if len(s) % self.split_size else 0
        bitesize = self.max_len * 2 * self.split_size
        sec = iter_utils.section(s[:bitesize], self.split_size)
        end = s[bitesize:]
        end36 = iter_utils.keep_many(end, string.ascii_letters + string.digits)
        q = int(end36, 36) if end36 else 0
        ds = int(self.dash)
        if q < ds * 2 // 3 or not q or q < 0:
            q = ds
        L = self.mag_num * z * q
        v = ''.join([str(hash(end))] + [str(hash(s) * L
                ).replace('-', self.dash) for s in sec])
        r = int(v)
        return sane_hex(r)[:self.max_len]


class PD13(PD12):
    '''
    Similar to PD10() but based on PD12().
    
    AUTHORS:
    v0.3.6+             --> pydsigner
    '''
    max_len = 32
    mag_num = 3
    split_size = 4
    dash = str(int(dec(13).exp()))


class PD14(PD12):
    '''
    Similar to PD11() but based on PD12().
    
    AUTHORS:
    v0.3.6+             --> pydsigner
    '''
    max_len = 64
    mag_num = 5
    split_size = 3
    dash = str(int(dec(17).exp()))


class RD(object):
    '''
    Seeds the random module with the object to be hashed; returns the str()ed 
    result of random.randint(self.small, self.big). Possible weaknesses: Does 
    random return the same numbers on all platforms? Will threading break 
    this?
    
    AUTHORS:
    v0.4.7+             --> pydsigner
    '''
    small = 0
    big = 10 ** 10
    
    def encode(self, s):
        random.seed(s)
        return str(random.randint(self.small, self.big))


class SHA1(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.sha1(s).hexdigest()


class SHA224(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.sha224(s).hexdigest()


class SHA256(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.sha256(s).hexdigest()


class SHA384(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.sha384(s).hexdigest()


class SHA512(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.sha512(s).hexdigest()


class MD5(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return hashlib.md5(s).hexdigest()


class Base16(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return base64.b16encode(s)
    def decode(self, s):
        return base64.b16decode(s)


class Base32(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return base64.b32encode(s)
    def decode(self, s):
        return base64.b32decode(s)


class Base64(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return base64.b64encode(s)
    def decode(self, s):
        return base64.b64decode(s)


class Hex(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return binascii.b2a_hex(s)
    def decode(self, s):
        return binascii.a2b_hex(s)


class HQX(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return binascii.b2a_hqx(s)
    def decode(self, s):
        return binascii.a2b_hqx(s)[0]


class UU(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return binascii.b2a_uu(s)
    def decode(self, s):
        return binascii.a2b_uu(s)


class URL(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    def encode(self, s):
        return urler.quote(s)
    def decode(self, s):
        return urler.unquote(s)


class HTML(object):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    conv_dict = {'<': '&#60;', '>': '&#62;', '"': '&#34;', '&': '&#38;'}
    
    def encode(self, s):
        return iter_utils.many(s, self.conv_dict)
    def decode(self, s):
        return iter_utils.many(s, self.conv_dict, True)


class CRC32(object):
    '''
    AUTHORS:
    v0.2.7+             --> pydsigner
    '''
    def encode(self, s):
        return zlib.crc32(s)


class Adler32(object):
    '''AUTHORS:
    v0.2.7+             --> pydsigner'''
    def encode(self, s):
        return zlib.adler32(s)


encoder_classes = dict(triplets=Triplets, pd2=PD2, pd3=PD3, pd4=PD4, pd5=PD5, 
                       pd6=PD6, pd7=PD7, pd8=PD8, pd9=PD9, pd10=PD10, 
                       pd11=PD11, pd12=PD12, pd13=PD13, pd14=PD14, rd=RD, 
                       sha1=SHA1, sha224=SHA224, sha256=SHA256, sha384=SHA384, 
                       sha512=SHA512, md5=MD5, base16=Base16, base32=Base32, 
                       base64=Base64, hex=Hex, hqx=HQX, uu=UU, url=URL, 
                       html=HTML, crc32=CRC32, adler32=Adler32)


def fetcher(encoder):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    return encoder_classes[encoder.lower()]()


def rand_key(L=10):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    res = ''
    while len(res) < L:
        v = str(math.log((random.randint(1, 33) * math.pi) ** 2))
        res += iter_utils.remove_many(v, 'e-.')
    return res[:L]


def multi_pass(user, pswd, times=1000, hasher=SHA512()):
    '''
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    for i in range(times):
        pswd = hasher.encode(pswd + user + str(i))
    return pswd



"Pydsigner's Generic Python Utilities (PGPU) -- a collection of handy modules" 
'''and packages for Python.

__init__        --  version information
compatibility   --  module to make it easier to write modules compatible with 
                    both 2.x and 3.x
iter_utils      --  generic iterable utilities
file_utils      --  file utility module
math_utils      --  utilities to work with bases plus some trig utilities
security        --  security and encoding module
time_widgets    --  tkinter time widgets
tk_utils        --  tkinter widgets and helper functions
wrappers        --  utility value wrappers for places requiring functions

tkinter2x       --  2.x and 3.x compatibility layer for tkinter


This file provides version information.

AUTHORS:
v0.2.0+             --> pydsigner
v1.0.0+             --> pydsigner
v1.0.3+             --> pydsigner
'''

__version__ = '1.1.0'



'''
File utility module.

AUTHORS:
v0.2.0+             --> pydsigner
'''

import os


FAIL = 0
SKIP = 1


def parse_conf(lines):
    '''
    A simple configuration file parser.
    this parser supports comments and multiple values assigned to one key. One
    possible gotcha is that every item will be returned as a list.
    
    >>> parse_conf(["q 1, 2, 3", "d   w", "# doesn't do anything"])
    {"q": ["1", "2", "3"], "d": ["w"]}
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    dt = {}
    for L in lines:
        if L.startswith('#'):
            continue
        p = L.split(None, 1)
        dt[p[0]] = p[1].split(', ')
    return dt


def make_conf(data):
    '''
    A simple dict-to-configuration-file converter. designed to work with
    parse_conf(lines) above.
    Example:
    
    make_conf({"q": ["1", "2", "3"], "d": ["w"]})
    ["q 1, 2, 3", "d   w"]
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    v1.0.0+             --> pydsigner
    '''
    return (k + '\t' + ', '.join(data[k]) for k in data)


def size_of_dir(directory, error_handling=SKIP):
    '''
    Returns the overall size of directory @directory. @error_handling 
    determines whether OSError()'s will cause the function to fail.
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    cwd = os.getcwd()
    d = os.path.abspath(directory)
    os.chdir(d)
    size = os.path.getsize(d)
    for f in os.listdir(d):
        try:
            if os.path.isdir(f):
                if not os.path.islink(f):
                    size += size_of_dir(f)
            else:
                if not os.path.islink(f):
                    size += os.path.getsize(f)
        
        except OSError:
            if error_handling == FAIL:
                raise
    os.chdir(cwd)
    return size


def exists_in_path(f, case=True):
    '''
    AUTHORS:
    v0.2.10+            --> pydsigner
    '''
    for p in os.environ['PATH'].split(os.path.pathsep):
        for i in os.listdir(p):
            if not case:
                if i.lower() == f.lower():
                    return p
            else:
                if i == f:
                    return p



'''
Utility wrappers for places requiring functions, but where defining a new 
function would be overkill, lambdas would be plain annoying, and/or 
state-retention is desired.

AUTHORS:
0.3.9+              --> pydsigner
'''

class ValueWrapper(object):
    '''
    Makes a value look like a function. Use where a function is required to 
    return a value, but only one value is desired. If a reference is stored, a 
    new value could be set later.
    
    >>> v = ValueWrapper(5)
    >>> v()
    5
    >>> v.value = 7
    >>> v()
    7
    
    AUTHORS:
    0.3.9+              --> pydsigner
    '''
    def __init__(self, value):
        self.value = value
    def __call__(self):
        return self.value


class BumpWrapper(object):
    '''
    Similar to ValueWrapper(), but increments the value every time it is 
    called.
    
    >>> b = BumpWrapper(10, 4)
    >>> b()
    10
    >>> b()
    14
    >>> b()
    18
    
    AUTHORS:
    0.3.9+              --> pydsigner
    '''
    def __init__(self, value, inc):
        self.original_value = self.value = value
        self.inc = inc
        self.first = True
    
    def __call__(self):
        if self.first:
            self.first = False
        else:
            self.value += self.inc
        return self.value


class GiveCount(object):
    '''
    This wrapper is designed to go in places such as the "key" keyword 
    argument to max(), min(), and sort(). It counts the occurences of the 
    passed objects and returns that.
    
    >>> max([1, 2, 3, 3, 2, 1, 1], key=GiveCount())
    1
    >>> counter = GiveCount()
    >>> counter(GiveCount)
    1
    >>> counter(3)
    1
    >>> counter(GiveCount)
    2
    >>> # A special feature for special applications
    ... counter(GiveCount, -1)
    1
    
    AUTHORS:
    0.3.10+             --> pydsigner
    '''
    def __init__(self, inc=1):
        self.d = {}
        self.inc = inc
    
    def __call__(self, obj, inc=None):
        v = inc if inc != None else self.inc
        if obj in self.d:
            v += self.d[obj]
        self.d[obj] = v
        return v
    
    def reset(self, inc=None):
        self.d = {}
        self.inc = inc if inc != None else self.inc
    
    def get_count(self, obj):
        self(obj, 0)
        return self.d[obj]


class CallWrapper(object):
    '''
    CallWrapper() is a decorator generator for decorators that wrap calls to 
    a function in another function. It has special features such as changing 
    the docstring to show help for the first function.
    
    AUTHORS:
    0.4.8+              --> pydsigner
    '''
    def __init__(self, funcs=[]):
        '''
        Create a CallWrapper() with @funcs for callables. They will be called 
        in order, so that funcs[0] will be called with any passed variables, 
        and then funcs[1] will be called on the result of that and so on.
        '''
        self.funcs = funcs
        self.__doc__ = funcs[0].__doc__
    
    def __repr__(self):
        return 'CallWrapper([%s])' % ', '.join(repr(r) for r in self.funcs)
    
    def __call__(self, *args, **kw):
        res = self.funcs[0](*args, **kw)
        for f in self.funcs[1:]:
            res = f(res)
        return res



'''
Utilities to work with bases, some trig utilities, a more powerful
version of decimal.Decimal(), a Vector() class, and just some random 
mathematicical utilities.

AUTHORS:
v0.2.0+             --> pydsigner
'''

import string
import math
from fractions import Fraction as frac
import cmath
import operator
import decimal

from .iter_utils import keep_many, flatten
from .compatibility import range

# should we use string.ascii_uppercase?
DEFAULT_REP_ORDER = string.digits + string.ascii_lowercase


def convert_to_base(b10int, base):
    '''
    Convert base 10 integer @b10int to base @base.
    
    >>> convert_to_base(400, "10", "9876543210")
    "699"
    >>> convert_to_base("10", 2)
    "1010"
    >>> convert_to_base(340.4, 30)
    "ab"
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    # We aren't going to try to convert floats. (Todo or fail?)
    uint = int(b10int)
    # We certainly aren't going to use float bases! (Fail?)
    ubase = abs(int(base))  
    res = ''
    sign = ''
    if uint < 0:
        uint = abs(uint)
        sign = '-'
    elif uint == 0:
        return DEFAULT_REP_ORDER[0]
    while 1:
        res = DEFAULT_REP_ORDER[uint % ubase] + res
        uint //= ubase
        if uint == 0:
            break
    return sign + res


def sgp_with_base(b10int, base):
    '''
    SGP stands for smallest greater power. This function returns and number
    so that @base to the return value (i)-th power is greater than @b10int
    and @base ** (i - 1) is not.
    
    >>> sgp_with_base(3256, 5)
    6
    >>> 6 ** 5
    7776
    >>> 6 ** 4
    1296

    AUTHORS:
    v0.2.0+             --> pydsigner
    v0.3.8+             --> ffao
    '''
    return int(math.log(b10int, base)) + 1


def legs(hyp, ratio=(1, 1)):
    '''
    Calculates the leg lengths of a triangle with a hypotenuse of @hyp where
    the side length ratio is @ratio. Nice for calculating screen dimensions.
    
    >>> legs(15, [9, 16])
    (7.353918594488384, 13.073633056868239)
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    r = float(ratio[0]) / ratio[1]
    L2 = hyp / math.sqrt(r ** 2 + 1)
    L1 = L2 * r
    return L1, L2


def euclidean_dist(c1, c2):
    '''
    Returns the Euclidean distance between coordinate @c1 and @c2.
    
    >>> euclidean_dist((4, 6), (9, -10))
    16.76305461424021
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    return math.hypot(c1[0] - c2[0], c1[1] - c2[1])


def sane_hex(v):
    '''
    Returns a more usable hex representation of int()'ed @v.
    
    >>> sane_hex(-846)
    '-34e'
    
    AUTHORS:
    v0.3.3+             --> pydsigner
    '''
    i = int(v)
    sign = '-' if i < 0 else ''
    i = abs(i)
    h = keep_many(hex(i), string.hexdigits).lstrip('0') or '0'
    return sign + h


def limit(val, bottom=None, top=None):
    '''
    Will return a copy of @val in such a way that:
    If @bottom is not None, the result will be no less than @bottom;
    If @top is not None, the result will be no more than @top.
    
    >>> limit(10)
    10
    >>> limit(10, 1)
    10
    >>> limit(10, 15)
    15
    >>> limit(10, 1, 8)
    8
    >>> # Can be used with anything that can be compared using max() and min()
    ... limit('6', top='3')
    '3'
    
    AUTHORS:
    v0.4.5+             --> pydsigner
    '''
    uval = val
    if bottom is not None:
        uval = max(uval, bottom)
    if top is not None:
        uval = min(uval, top)
    return uval


def pascals_triangle(depth):
    '''
    Calculate Pascal's triangle to @depth places. returns a list of lists.
    
    >>> pascals_triangle(5)
    [[1], [1, 1], [1, 2, 1], [1, 3, 3, 1], [1, 4, 6, 4, 1]]
    
    AUTHORS:
    v0.4.7+             --> pydsigner
    '''
    res = []
    for i in range(depth):
        row = [1]
        for c in range(1, i + 1):
            row.append(sum(res[-1][c - 1:c + 1]))
        res.append(row)
    return res


def to_polar(vec):
    '''
    Returns rectangular vector @vec as a polar coordinate represented as a 
    (distance, angle) tuple.
    
    AUTHORS:
    v0.4.7+             --> pydsigner
    '''
    return cmath.polar(complex(vec[0], vec[1]))


def to_vector(polar):
    '''
    Returns polar coordinate tuple @polar as a Vector().
    
    AUTHORS:
    v0.4.7+             --> pydsigner
    '''
    i = cmath.rect(*polar)
    return Vector(i.real, i.imag)


def rotate_vector(vec, degrees):
    '''
    Rotates rectangular vector @vec @degrees.
    NOTE: Use Vector().rotated() instead, which this simply wraps!
    
    AUTHORS:
    v0.4.7+             --> pydsigner
    v1.0.0+             --> pydsigner
    '''
    return Vector(vec).rotated(degrees)


def factor(n):
    '''
    Returns a list of primes (excluding 1) which, when multiplied together, 
    equal @n.
    NOTE: This is not the same as factors() below, which returns every whole
    number that can divide into @n evenly.
    NOTE: If @n is prime, this function returns an empty list.
    
    >>> factor(5)
    []
    >>> factor(34)
    [2, 17]
    >>> factor(132)
    [2, 2, 3, 11]
    
    AUTHORS:
    v1.0.4+             --> pydsigner
    '''
    # NOTE: Could do a primes check, but that would probably be about as 
    # expensive.
    # TODO: is there a shortcut for this?
    ceil = n // 2 + 1
    factors = []
    i = 2
    # Note that, if @n is 1, this code will never run. So a hardwire doesn't 
    # seem to make sense.
    while i < ceil:
        nn, rem = divmod(n, i)
        if rem:
            i += 1
        else:
            n = nn
            factors.append(i)
    
    return factors


def ifactor(n):
    '''
    The same as factor() above, but designed as a generator; this function 
    should be better for massive numbers.
    
    AUTHORS:
    v1.0.4+             --> pydsigner
    '''
    ceil = n // 2 + 1
    i = 2
    while i < ceil:
        nn, rem = divmod(n, i)
        if rem:
            i += 1
        else:
            n = nn
            yield i


def factors(n):
    '''
    Returns an set of every factor of @n (including 1 and @n).
    
    >>> factors(36)
    set([1, 2, 3, 4, 6, 9, 12, 18, 36])

    AUTHORS:
    v0.4.9+             --> pydsigner
    '''
    return set(x for x in range(1, n // 2 + 1) if not n % x) | set((n,))


def polyroots(q, p, pol):
    '''
    Finds all of the roots of polynomial @pol, with the constant of highest 
    degree @q and the constant of x**0 @p. Returns a set of all the roots.
    
    >>> polyroots(1, 2, 'x**3 - 2*x**2 - x + 2')
    set([1, 2, -1])
    
    TODO: Add ability to find @q and @p from @pol
    
    AUTHORS:
    v0.5.1+             --> pydsigner
    '''
    # First a set of fractions is generated from the factors of @q and @p;
    # These fractions are every possible (x/y equaling -x/-y and 
    # -x/y equaling x/-y) combination of these factors (the factors of @p being 
    # in the denominator) that makes the polynomial equal 0
    S = set(x for x in 
      set(flatten((frac(pv, qv), frac(-pv, qv)) for qv in factors(abs(q)) 
                   for pv in factors(abs(p)))) 
      if eval(pol.replace('x', 'frac("%s")' % x)) == 0)
    
    # Make new set where whole numbers are changed to int()s and any lossless 
    # conversions to float()s are made
    nset = set()
    for n in S:
        if n.denominator == 1:
            nset.add(n.numerator)
        elif frac(float(n)) == n:
            nset.add(float(n))
        else:
            nset.add(n)
    return nset


class ExtendedDecimal(decimal.Decimal):
    '''
    An Extended version of decimal.Decimal(), with more geometric features,
    similar to those found in the standard math module. These features
    currently are:
        pi() (slightly modified from the decimal documentation)
        cos() (ibid)
        sin() (ibid)
        radians()
        degrees()
        tan()

    AUTHORS:
    v0.4.0+             --> pydsigner
    '''
    def __repr__(self):
        '''
        eval(repr(exdec)) <--> exdec
        
        AUTHORS:
        v0.4.1+             --> pydsigner
        '''
        return 'ExtendedDecimal(\'%s\')' % str(self)

    def pi(self):
        '''
        Compute Pi to the current precision.
        
        AUTHORS:
        v0.4.0+             --> pydsigner
        '''
        decimal.getcontext().prec += 2  # extra digits for intermediate steps
        three = decimal.Decimal(3)
        lasts, t, s, n, na, d, da = 0, three, 3, 1, 0, 0, 24
        while s != lasts:
            lasts = s
            n, na = n+na, na+8
            d, da = d+da, da+32
            t = (t * n) / d
            s += t
        decimal.getcontext().prec -= 2  # reset precision
        return ExtendedDecimal(s)       # don't return a Decimal() instance!

    def cos(self):
        '''
        Return the cosine of self as measured in radians.
        
        AUTHORS:
        v0.4.0+             --> pydsigner
        '''
        decimal.getcontext().prec += 2
        i, lasts, s, fact, num, sign = 0, 0, 1, 1, 1, 1
        while s != lasts:
            lasts = s
            i += 2
            fact *= i * (i-1)
            num *= self * self
            sign *= -1
            s += num / fact * sign
        decimal.getcontext().prec -= 2
        return ExtendedDecimal(+s)

    def sin(self):
        '''
        Return the sine of self as measured in radians.
        
        AUTHORS:
        v0.4.0+             --> pydsigner
        '''
        decimal.getcontext().prec += 2
        i, lasts, s, fact, num, sign = 1, 0, self, 1, self, 1
        while s != lasts:
            lasts = s
            i += 2
            fact *= i * (i-1)
            num *= self * self
            sign *= -1
            s += num / fact * sign
        decimal.getcontext().prec -= 2
        return ExtendedDecimal(s)

    def tan(self):
        '''
        Returns the tangent of self as measured in radians.
        
        AUTHORS:
        v0.4.1+             --> pydsigner
        '''
        return ExtendedDecimal(self.sin() / self.cos())

    def radians(self):
        '''
        returns self (as in degrees) in radians.
        AUTHORS:
        v0.4.0+             --> pydsigner
        '''
        return ExtendedDecimal(self * self.pi() / 180)

    def degrees(self):
        '''
        returns self (as in radians) in degrees.
        AUTHORS:
        v0.4.0+             --> pydsigner
        '''
        return ExtendedDecimal(self * 180 / self.pi())


class Vector(object):
    '''
    2d vector class, supports vector and scalar operators, and also provides
    a bunch of high level functions.
    NOTE: This was taken from Eli Bendersky, who took it from the Pygame Wiki
    (http://pygame.org/wiki/2DVectorClass). It was also slightly modified to be
    cross version compatible, be more sane, fit in with pgpu, and be more 
    useful. I opted for this implementation over my own because this was more 
    complete.
    
    AUTHORS:
    v0.4.3+             --> Pygame Wiki/pydsigner
    v1.0.0+             --> Pygame Wiki/pydsigner
    '''
    __slots__ = ['x', 'y']
    def __init__(self, x_or_pair=None, y=None):
        if y == None:
            if x_or_pair == None:
                self.x = self.y = 0
            else:
                self.x = x_or_pair[0]
                self.y = x_or_pair[1]
        else:
            self.x = x_or_pair
            self.y = y

    def __len__(self):
        return 2
    def __iter__(self):
        return iter((self.x, self.y))

    def __getitem__(self, key):
        if key == 0:
            return self.x
        elif key == 1:
            return self.y
        else:
            raise IndexError('Invalid subscript %s to Vector' % key)

    def __setitem__(self, key, value):
        if key == 0:
            self.x = value
        elif key == 1:
            self.y = value
        else:
            raise IndexError('Invalid subscript %s to Vector' % key)

    # String representation (for debugging)
    def __repr__(self):
        return 'Vector(%s, %s)' % (self.x, self.y)

    # Comparison
    def __eq__(self, other):
        if hasattr(other, '__getitem__') and len(other) == 2:
            return self.x == other[0] and self.y == other[1]
        else:
            return False

    def __ne__(self, other):
        if hasattr(other, '__getitem__') and len(other) == 2:
            return self.x != other[0] or self.y != other[1]
        else:
            return True

    def __nonzero__(self):
        return self.x or self.y
    __bool__ = __nonzero__      # for those poor 3.x users ;-) -- pydsigner

    # Generic operator handlers
    def _o2(self, other, f):
        '''
        Any two-operator operation where the left operand is a Vector
        '''
        if isinstance(other, Vector):
            return Vector(f(self.x, other.x),
                         f(self.y, other.y))
        elif (hasattr(other, '__getitem__')):
            return Vector(f(self.x, other[0]),
                         f(self.y, other[1]))
        else:
            return Vector(f(self.x, other),
                         f(self.y, other))

    def _r_o2(self, other, f):
        '''
        Any two-operator operation where the right operand is a Vector
        '''
        if (hasattr(other, '__getitem__')):
            return Vector(f(other[0], self.x),
                         f(other[1], self.y))
        else:
            return Vector(f(other, self.x),
                         f(other, self.y))

    def _io(self, other, f):
        '''
        inplace operator
        '''
        if (hasattr(other, '__getitem__')):
            self.x = f(self.x, other[0])
            self.y = f(self.y, other[1])
        else:
            self.x = f(self.x, other)
            self.y = f(self.y, other)
        return self

    # Addition
    def __add__(self, other):
        return self._o2(other, operator.add)
    def __radd__(self, other):
        return self._r_o2(other, operator.add)
    def __iadd__(self, other):
        return self._io(other, operator.add)

    # Subtraction
    def __sub__(self, other):
        return self._o2(other, operator.sub)
    def __rsub__(self, other):
        return self._r_o2(other, operator.sub)
    def __isub__(self, other):
        return self._io(other, operator.sub)

    # Multiplication
    def __mul__(self, other):
        return self._o2(other, operator.mul)
    def __rmul__(self, other):
        return self._r_o2(other, operator.mul)
    def __imul__(self, other):
        return self._io(other, operator.mul)

    # Division
    def __div__(self, other):
        return self._o2(other, operator.div)
    def __rdiv__(self, other):
        return self._r_o2(other, operator.div)
    def __idiv__(self, other):
        return self._io(other, operator.div)

    def __floordiv__(self, other):
        return self._o2(other, operator.floordiv)
    def __rfloordiv__(self, other):
        return self._r_o2(other, operator.floordiv)
    def __ifloordiv__(self, other):
        return self._io(other, operator.floordiv)

    def __truediv__(self, other):
        return self._o2(other, operator.truediv)
    def __rtruediv__(self, other):
        return self._r_o2(other, operator.truediv)
    def __itruediv__(self, other):
        return self._io(other, operator.truediv)

    # Modulo
    def __mod__(self, other):
        return self._o2(other, operator.mod)
    def __rmod__(self, other):
        return self._r_o2(other, operator.mod)

    def __divmod__(self, other):
        return self._o2(other, operator.divmod)
    def __rdivmod__(self, other):
        return self._r_o2(other, operator.divmod)

    # Exponentation
    def __pow__(self, other):
        return self._o2(other, operator.pow)
    def __rpow__(self, other):
        return self._r_o2(other, operator.pow)

    # Bitwise operators
    def __lshift__(self, other):
        return self._o2(other, operator.lshift)
    def __rlshift__(self, other):
        return self._r_o2(other, operator.lshift)

    def __rshift__(self, other):
        return self._o2(other, operator.rshift)
    def __rrshift__(self, other):
        return self._r_o2(other, operator.rshift)

    def __and__(self, other):
        return self._o2(other, operator.and_)
    __rand__ = __and__

    def __or__(self, other):
        return self._o2(other, operator.or_)
    __ror__ = __or__

    def __xor__(self, other):
        return self._o2(other, operator.xor)
    __rxor__ = __xor__

    # Unary operations
    def __neg__(self):
        return Vector(operator.neg(self.x), operator.neg(self.y))

    def __pos__(self):
        return Vector(operator.pos(self.x), operator.pos(self.y))

    def __abs__(self):
        return Vector(abs(self.x), abs(self.y))

    def __invert__(self):
        return Vector(-self.x, -self.y)

    # vectory functions
    def get_length_sqrd(self):
        return self.x ** 2 + self.y ** 2

    def get_length(self):
        return math.sqrt(self.x ** 2 + self.y ** 2)
    
    def __setlength(self, value):
        length = self.get_length()
        self.x *= value/length
        self.y *= value/length
    length = property(get_length, __setlength, None, 
                'gets or sets the magnitude of the vector')

    def rotate(self, angle_degrees):
        radians = math.radians(angle_degrees)
        cos = math.cos(radians)
        sin = math.sin(radians)
        x = self.x * cos - self.y * sin
        y = self.x * sin + self.y * cos
        self.x = x
        self.y = y

    def rotated(self, angle_degrees):
        radians = math.radians(angle_degrees)
        cos = math.cos(radians)
        sin = math.sin(radians)
        x = self.x * cos - self.y * sin
        y = self.x * sin + self.y * cos
        return Vector(x, y)

    def get_angle(self):
        if (self.get_length_sqrd() == 0):
            return 0
        return math.degrees(math.atan2(self.y, self.x))
    
    def __setangle(self, angle_degrees):
        self.x = self.length
        self.y = 0
        self.rotate(angle_degrees)
    angle = property(get_angle, __setangle, None, 
                'gets or sets the angle of a vector')

    def get_angle_between(self, other):
        cross = self.x * other[1] - self.y * other[0]
        dot = self.x * other[0] + self.y * other[1]
        return math.degrees(math.atan2(cross, dot))

    def normalized(self):
        length = self.length
        if length != 0:
            return self / length
        return Vector(self)

    def normalize_return_length(self):
        length = self.length
        if length != 0:
            self.x /= length
            self.y /= length
        return length

    def perpendicular(self):
        return Vector(-self.y, self.x)

    def perpendicular_normal(self):
        length = self.length
        if length != 0:
            return Vector(-self.y / length, self.x / length)
        return Vector(self)

    def dot(self, other):
        return float(self.x * other[0] + self.y * other[1])

    def get_distance(self, other):
        return euclidean_dist(self, other)

    def get_dist_sqrd(self, other):
        return (self.x - other[0])**2 + (self.y - other[1])**2

    def projection(self, other):
        other_length_sqrd = other[0] ** 2 + other[1] ** 2
        proj_length_times_other_length = self.dot(other)
        return other * (proj_length_times_other_length / other_length_sqrd)

    def cross(self, other):
        return self.x*other[1] - self.y*other[0]

    def interpolate_to(self, other, range):
        return Vector(self.x + (other[0] - self.x) * range, 
                self.y + (other[1] - self.y) * range)

    def convert_to_basis(self, x_vector, y_vector):
        return Vector(self.dot(x_vector) / x_vector.get_length_sqrd(), 
                self.dot(y_vector) / y_vector.get_length_sqrd())

    def __getstate__(self):
        return [self.x, self.y]

    def __setstate__(self, dict):
        self.x, self.y = dict



'''
A set of dict()-like classes with differing features.

AUTHORS:
v0.4.5              --> pydsigner
'''

from .compatibility import range

try:
    from . import tkinter2x as tk
    from .tkinter2x.constants import *
    from .tkinter2x.simpledialog import askstring
except ImportError:
    # No tkinter, don't use the GUIDict!
    pass


class SortedDict(object):
    '''
    A sorted dictionary that is implemented as a merger of a dict() and a 
    list(). It exposes more of the dict() style via overloading but the list() 
    style is also accessible via named methods. When in doubt about which 
    "side" a method accesses, look at the signature; if one of the arguments is 
    @key, then it would access the dict() side. A method with @index as an 
    argument would access the list() side.
    
    AUTHORS:
    v0.4.5+             --> pydsigner
    '''
    def __init__(self, *args):
        '''
        SortedDict([(key_0, value_0), (key_1, value_1)..., (key_n, value_n)])
        SortedDict([key_0, key_1..., key_n], [value_0, value_1..., value_n])
        SortedDict([key_0, key_1..., key_n], default)
        SortedDict((key_0, value_0), (key_1, value_1)..., (key_n, value_n))
        SortedDict()
        '''
        self.data = []
        self.keydict = {}
        if len(args) == 1:
            for key, value in args[0]:
                self._add(key, value)
        elif len(args) == 2:
            if hasattr(args[1], '__len__') and len(args[1]) == len(args[0]):
                for i in range(len(args[0])):
                    self._add(args[0][i], args[1][i])
            else:
                for key in args[0]:
                    self._add(key, args[1])
        else:
            for key, value in args:
                self._add(key, value)
    
    def __delitem__(self, key):
        '''
        Delete item @key from the dictionary.
        '''
        del self.keydict[key]
    
    def __iter__(self):
        '''
        Returns an iterator over the dictionary\'s keys.
        '''
        return iter(self.keys())
    
    def _content_rep(self):
        return ', '.join(repr(pair) for pair in self.items())
    def __str__(self):
        return '{}' % self._content_rep()
    def __repr__(self):
        return 'SortedDict(%s)' % self._content_rep()
    
    def get(self, key, *arg):
        '''
        get(key, [default])
        Get the value for @key from the dictionary. If @key is not in the 
        dictionary and @default is supplied, return @default.
        '''
        if len(arg) > 1:
            raise TypeError('Expected at most 2 arguments, got %s' 
                             % (len(arg) + 1))
        elif len(arg) == 1:
            ret = self.keydict.get(key, None)
            return self.data[ret] if ret != None else arg[0]
        else:
            return self.data[self.keydict.get(key)]
    __getitem__ = get
    
    def set(self, key, value):
        '''
        Sets the value for @key to @value.
        '''
        k = self.keydict.get(key)
        if k == None:
            self._add(key, value)
        else:
            self.data[k] = value
    __setitem__ = set
    
    def get_at(self, index):
        '''
        Get the value at @index. Probably quite slow, as the list() side of 
        the dictionary is not favored.
        '''
        return self.data[self.keydict[self.keys()[index]]]
    
    def set_at(self, index, value):
        '''
        Set the value at @index to @value. Probably quite slow, as the 
        list() side of the dictionary is not favored.
        '''
        self.data[self.keydict[self.keys()[index]]] = value
    def delete_at(self, index):
        '''
        Delete the value at index.Probably quite slow, as the list() side 
        of the dictionary is not favored.
        '''
        for p in self.keydict.items():
            if p[1] == index:
                the_pair = p
                break
        del self.keydict[the_pair[0]]
    
    def keys(self):
        return sorted(self.keydict.keys(), key = self._sorter)
    def values(self):
        return [self.data[self.keydict[k]] for k in self.keys()]
    def items(self):
        return zip(self.keys(), self.values())
    
    def rebuild(self):
        '''
        Remove zombie data from the data store to free up memory.
        '''
        vals = self.keydict.values()
        slen = len(self.data)
        for i, v in enumerate(self.data):
            if not v in vals:
                del self.data[i]
        # Check to see if any changes were made before rebuilding
        if slen != len(self.data):
            self.__init__(vals, self.data)
    
    def _sorter(self, key):
        return self.keydict.get(key)
    
    def _add(self, key, value):
        self.keydict[key] = len(self.keydict)
        self.data.append(value)


class UpdatingDict(dict):
    '''
    A dictionary that will notify about changes.
    
    AUTHORS:
    v0.4.5+             --> pydsigner
    '''
    def __init__(self, *args, **kw):
        dict.__init__(self, *args, **kw)
        self.n = []
    
    def __delitem__(self, key):
        dict.__delitem__(self, key)
        [obj.update() for obj in self.n]
    
    def set(self, key, value):
        dict.__setitem__(self, key, value)
        [obj.update() for obj in self.n]
    __setitem__ = set
    
    def add_notify(self, obj):
        '''
        Add @obj to the notification list.
        '''
        if obj not in self.n:
            self.n.append(obj)
    
    def del_notify(self, obj):
        '''
        Remove @obj from the notification list. Will raise AssertionError if 
        @obj is not in the notification list.
        '''
        assert obj in self.n, (
                'Cannot remove objects from notification list' + 
                ' that are not in it!')
        del self.n[self.n.index(obj)]


class GUIDictItem(tk.Frame):
    '''
    A basic item for the GUIDict() graphical implementation of a dictionary. 
    NB: This item converts everything to strings, so be careful where you use 
    it.
    
    AUTHORS:
    v0.4.5+             --> pydsigner
    '''
    def __init__(self, master, key, value, *args, **kw):
        tk.Frame.__init__(self, master, *args, **kw)
        self.var = tk.StringVar()
        self.var.set(value[0])
        self.key = key
        
        tk.Label(self, text = self.key).grid(row=0, column=0, sticky=W + E)
        tk.Entry(self, textvariable = self.var
                  ).grid(row=0, column=1, sticky=W + E)
        tk.Button(self, command = self.update, text='Update'
                  ).grid(row=0, column=2, sticky=W + E)
        tk.Button(self, command = self.deleter, text='Delete'
                  ).grid(row=0, column=3, sticky=W + E)
        
        self.columnconfigure(0, weight=6)
        self.columnconfigure(1, weight=12)
        self.columnconfigure(2, weight=3)
        self.columnconfigure(3, weight=1)
    
    def update(self):
        self.master.set(self.key, [self.var.get()])
    def deleter(self):
        self.master.delete_key(self.key)


class GUIDict(tk.Frame):
    '''
    A basic GUI implementation of a dictionary.
    
    AUTHORS:
    v0.4.5+             --> pydsigner
    '''
    item = GUIDictItem
    def __init__(self, master, dictobj=UpdatingDict(), *args, **kw):
        tk.Frame.__init__(self, master, *args, **kw)
        bbox = tk.Frame(self)
        tk.Button(bbox, text='Add key', command=self.adder
                  ).pack(side=RIGHT, expand=True, fill=X)
        bbox.pack(side=BOTTOM, expand=True, fill=X)
        self.dictobj = dictobj
        self.dictobj.add_notify(self)
        self.update()
    
    def __str__(self):
        return str(self.dictobj)
    def __repr__(self):
        return repr(self.dictobj)
    
    def update(self):
        try:
            for item in self.reps.values():
                item.destroy()
        except:
            pass
        
        self.reps = {}
        for k in sorted(self.dictobj):
            self.reps[k] = self.item(self, k, self.dictobj.get(k))
            self.reps[k].pack(side = TOP, fill = X, expand = True)
    
    def set(self, key, value):
        self.dictobj.set(key, value)
    
    def delete_key(self, key):
        del self.dictobj[key]
    
    def adder(self):
        r = askstring('Add key', 'Enter the key to add:')
        if r:
            self.dictobj.set(r, [''])



'''
Tkinter widgets and helper functions.

AUTHORS:
v0.2.0+             --> pydsigner
'''

from . import tkinter2x as tk
from .tkinter2x.font import families
from .tkinter2x.constants import *
from .tkinter2x import scrolledtext as stext


OLDSTYLE_FONTS = ['ShadowedBlack', 'Benighted', 'Old English Text']
FLOWING_FONTS = ['Palace Script MT', 'Bradley Hand ITC', 'French Script',
                 'French Script MT', 'Edwardian Script ITC', 'Blackadder ITC',
                 'Script Bold', 'Script MT Bold', 'Zola', 'Vivaldi']
SEMIFLOWING_FONTS = ['URW Chancery L']
SANS_FONTS = ['Liberation Sans', 'FreeSans', 'DejaVu Sans', 'Sans']
MONO_FONTS = ['Liberation Mono', 'DejaVu Sans Mono', 'Monospace', 'FreeMono']
ALLCAP_FONTS = ['Algerian', 'Castellar']


def best_font(master, fonts=[]):
    '''
    Returns the first font name in @fonts that tkinter can find. Pass a Tk() 
    instance to @master.
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    sysfonts = families(root=master)
    for font in fonts:
        if font in sysfonts:
            return font


class SList(tk.Frame):
    '''
    A basic scrolling list display based on Mark Lutz's ScrolledList() widget 
    from Programming Python, 3rd edition.
    
    AUTHORS:
    v1.0.2+             --> pydsigner/Mark Lutz
    '''
    def __init__(self, master):
        tk.Frame.__init__(self, master)
        
        sbar = tk.Scrollbar(self)
        l = tk.Listbox(self, relief=SUNKEN)
        
        sbar.config(command=l.yview)
        l.config(yscrollcommand=sbar.set)
        
        sbar.pack(side=RIGHT, fill=Y)
        l.pack(side=LEFT, expand=YES, fill=BOTH)
        
        self.listbox = l
    
    def set(self, l):
        '''
        Fill the listbox.
        '''
        self.listbox.delete(0, END)
        self.listbox.insert(END, *l)


class TextPlus(tk.Text):
    '''
    A version of Text() with additional methods for getting, setting, and 
    clearing the text contained by it. The afore mentioned methods are based on
    some from Mark Lutz's scrolledtext.ScrolledText() widget as seen in 
    Programming Python, 3rd Edition.
    
    AUTHORS:
    v0.4.4+             --> pydsigner/Mark Lutz
    '''
    def gettext(self):
        return self.get('1.0', END + '-1c')
    def settext(self, text):
        self.clear()
        self.insert('1.0', text)
    
    def clear(self):
        self.delete('1.0',END)


class STextPlus(stext.ScrolledText, TextPlus):
    '''
    A simple subclass of TextPlus() with scrolling.
    
    AUTHORS:
    v0.2.0+             --> pydsigner/Mark Lutz
    v0.4.4+             --> pydsigner
    '''


class Console(TextPlus):
    '''
    An extension of TextPlus() widget that additionally acts like a file. 
    However, due to the implications of being editable externally e.g. by 
    someone using an interface, Console() uses a cache that must be manually 
    refresh()ed.
    
    AUTHORS:
    v0.4.4+             --> pydsigner
    '''
    def __init__(self, master, *args, **kw):
        TextPlus.__init__(self, master, *args, **kw)
        self.refresh()
    
    def get_fresh(self):
        '''
        Returns False if the content of the widget has changed since the last 
        refresh(), otherwise it returns True.
        '''
        return self._oldcache == self.gettext()
    
    def refresh():
        '''
        Refresh the cache (equivalent to closing and opening a file).
        '''
        self._cache = self._oldcache = self.gettext()
    
    def peek(size=-1):
        '''
        Same as read, but does not affect the cache.
        '''
        val = self.read(size)
        self._cache = val + self._cache
        return val
    
    ### Standard file read methods
    
    def read(size=-1):
        if size < 0:
            size = len(self._cache)
        sz = int(size)
        res = self._cache[:sz]
        self._cache = self._cache[sz:]
        return res
    
    def readline(size=-1):
        maxdata = self.peek(size)
        pos = maxdata.find('\n')
        if pos != -1:
            return self.read(pos + 1)
        else:
            return self.read(size)
    
    def readlines(size=-1):
        '''
        Uses .readline() to get lines; if size is positive, the total data 
        size will be roughly bounded by it.
        '''
        res = []
        dsize = 0
        while 1:
            data = self.readline()
            dsize += len(data)
            if not data:
                return res
            res.append(data)
            if size > 0 and dsize >= size:
                return res
    
    ### Standard file write methods
    
    def write(self, data):
        self.insert(END + '-1c', data)
    
    def writelines(self, seq):
        for line in seq:
            self.write(line)
    
    xwritelines = writelines


class SConsole(stext.ScrolledText, Console):
    '''
    A simple subclass of Console() with scrolling.
    
    AUTHORS:
    v0.4.4+             --> pydsigner
    '''


class RadioBar(tk.Frame):
    '''
    A RadioBar mega widget. Borrows from Mark Lutz's demoRadio.Demo() widget 
    in Programming Python, 3rd Edition.
    
    AUTHORS:
    v0.2.0+             --> pydsigner/Mark Lutz
    '''
    def __init__(self, parent=None, default=None, picks=[], side=LEFT, 
                 anchor=W):
        '''
        Pass the desired master to @parent. If @default is not None, the radio 
        bar will initialized to it. @side and @anchor will be passed to the 
        packer for each RadioButton() created with a value from @picks.
        '''
        tk.Frame.__init__(self, parent)
        self.var = tk.StringVar()
        for pick in picks:
            rad = tk.Radiobutton(self, text=pick, value=pick, variable=self.var)
            rad.pack(side=side, anchor=anchor, expand=YES)
        if default != None:
            self.var.set(default)
    
    def state(self):
        return self.var.get()


class FontDialog(tk.Toplevel):
    '''
    A font picker megawidget.
    
    AUTHORS:
    v0.2.2+             --> pydsigner
    '''
    def __init__(self, defaults=('courier', 11, 'normal'), **kw):
        '''
        @default determines the default font. **@kw is passed to the 
        Toplevel() providing the window the font picker is displayed in.
        '''
        tk.Toplevel.__init__(self, **kw)
        
        self.title('Choose Font')
        self.protocol('WM_DELETE_WINDOW', self.cancel)
        
        self.preview = tk.Label(self, font=('courier', 15, 'bold'), 
                                text='Font')
        self.preview.pack(side=TOP)
        
        buttonbar = tk.Frame(self)
        
        tk.Button(buttonbar, text='Ok', command=self.ok).pack(side=LEFT)
        tk.Button(buttonbar, text='Cancel', command=self.cancel
                  ).pack(side=RIGHT)
        
        buttonbar.pack(side=BOTTOM)
        
        self.font = tk.StringVar()
        self.bold = tk.IntVar()
        self.size = tk.IntVar()
        self.italic = tk.IntVar()
        
        self.font.set(defaults[0])
        self.size.set(defaults[1])
        if 'bold' in defaults[2]:
            self.bold.set(1)
        if 'italic' in defaults[2]:
            self.italic.set(1)
        
        modbar = tk.Frame(self)
        
        mleft = tk.Frame(modbar)
        
        tk.OptionMenu(mleft, self.size, 7, 8, 9, 10, 11, 12, 13, 14, 16, 18, 
                      20, 22, 24).pack(side=TOP)
        tk.Checkbutton(mleft, variable=self.bold).pack(side=TOP)
        tk.Checkbutton(mleft, variable=self.italic).pack(side=TOP)
        mleft.pack(side=LEFT)
        
        mright = tk.Frame(modbar)
        tk.Label(mright, text='size').pack(side=TOP)
        tk.Label(mright, text='bold').pack(side=TOP)
        tk.Label(mright, text='italic').pack(side=TOP)
        mright.pack(side=LEFT)
        
        modbar.pack(side=RIGHT)
        
        fontbar = tk.Frame(self)
        tk.OptionMenu(fontbar, self.font, *sorted(self.fonts())).pack()
        fontbar.pack(side=LEFT)
        self.update()
        self.grab_set()
        self.focus_set()
        self.wait_window()
    
    def update(self):
        self.preview['font'] = self.getfont()
        self.after(100, self.update)
    
    def get_font(self):
        modifiers = ('bold ' if self.bold.get() else '' + 
                     'italic' if self.italic.get() else '').rstrip()
        if not modifiers:
            modifiers = 'normal'
        return (self.font.get(), self.size.get(), modifiers)
    
    def ok(self):
        self.result = self.get_font()
        self.destroy()
    
    def cancel(self):
        self.result = None
        self.destroy()
    
    def fonts(self):
        return [font.lower() for font in families(self)]


def ask_font(*args, **kw):
    '''
    Shortcut for using FontDialog(). *@args and **@kw will be passed to the 
    FontDialog() creator.
    
    AUTHORS:
    v0.2.2+             --> pydsigner
    '''
    return FontDialog(*args, **kw).result




'''
Module to make it easier to write code compatible with both 2.x and 3.x.

AUTHORS:
v0.2.0+             --> pydsigner
v1.0.1+             --> pydsigner
'''

__all__ = ['input', 'range', 'chr', 'str', 'Print']

try:
    input = raw_input
    range = xrange
    chr = unichr
    str = unicode
except NameError:
    input = input
    range = range
    chr = chr
    str = str

Print = __builtins__.get('print')



'''
Tkinter time widgets.

AUTHORS:
v0.2.2+                --> pydsigner
'''

from . import tkinter2x as tk
from .tkinter2x.constants import *
import time
import datetime


class DigitalClock(tk.Label):
    '''
    A digital clock display based on Label().
    
    AUTHORS:
    v0.2.2+             --> pydsigner
    '''
    def __init__(self, master=None, time_format='%H:%M:%S', **kw):
        '''
        @time_format is passed to time.strftime() to obtain the displayed 
        time look. **@kw is passed to the Label() constructor.
        '''
        self.time_format = time_format
        kw['text'] = self.givetime()
        tk.Label.__init__(self, master, **kw)
        self.setdisplay()
    
    def givetime(self):
        try:
            return time.strftime(self.time_format)
        except TypeError:
            return time.strftime('%I:%M:%S %p')
    
    def setdisplay(self):
        self['text'] = self.givetime()
        self.after(100, self.setdisplay)


class BasicChronograph(tk.Label):
    '''
    A basic chronograph widget that must be code controlled.
    
    AUTHORS:
    v0.2.2+             --> pydsigner
    '''
    def __init__(self, master=None, **kw):
        '''
        **@kw is passed to the Label() constructor.
        '''
        kw['text'] = '00:00:00.00'
        tk.Label.__init__(self, master, **kw)
        self.reset()
        self.update()
    
    def start(self):
        if not self.on:
            self.reset()
            self.on = True
    
    def stop(self):
        self.on = False
    
    def switch(self):
        self.on = not self.on
    
    def reset(self):
        self.on = False
        self.starttime = datetime.timedelta(seconds=time.time())
        self['text'] = '00:00:00.00'
        self.time_expired = datetime.timedelta()
    
    def update(self):
        timedif = (datetime.timedelta(seconds=time.time()) - 
                (self.starttime + self.time_expired))
        
        if self.on:
            self.time_expired += timedif
            hrs = str(self.time_expired.days * 24).zfill(2)
            mins = str(self.time_expired.seconds // 60).zfill(2)
            secs = str(self.time_expired.seconds % 60).zfill(2)
            hundrths = str(self.time_expired.microseconds // 10000).zfill(2)
            self['text'] = '%s:%s:%s.%s' % (hrs, mins, secs, hundrths)
        else:
            self.starttime += timedif
        
        self.after(20, self.update)


class Chronograph(tk.Frame):
    '''
    A more advanced chronograph using BasicChronograph(). This one has 
    external controls.
    
    AUTHORS:
    v0.2.2+             --> pydsigner
    '''
    def __init__(self, master=None, **kw):
        '''
        **@kw is passed to the BasicChronograph() constructor.
        '''
        tk.Frame.__init__(self, master)
        
        self.chron = BasicChronograph(self, **kw)
        self.chron.pack(side=TOP)
        
        self.bbar = tk.Frame(self)
        self.bbar.pack(side=BOTTOM)
        
        self.stopstart = tk.Button(self.bbar, text='start', command=self.flip)
        self.reset = tk.Button(self.bbar, text='reset', command=self.reset)
        self.stopstart.pack(side=LEFT)
        self.reset.pack(side=RIGHT)
    
    def flip(self):
        self.stopstart['text'] = 'start' if self.chron.on else 'stop'
        self.chron.switch()
    
    def reset(self):
        self.chron.reset()
        self.stopstart['text'] = 'start'



'''
Generic iterable utilities. These used to be contained in the main package.

AUTHORS:
v1.0.0+             --> pydsigner
'''

def replace_many(s, d, inverse=False):
    '''
    Goes through dict @d's keys and replaces their occurences with their 
    value. If @inverse is true, the values are replaced by the keys.
    NOTE: Results may vary from run to run and machine to machine because of 
    Python's dictionary optimization, especially if @inverse is true and some 
    keys have the same value. If @inverse is true, no empty strings can be 
    among the values, and if not, no empty string keys.

    >>> replace_many('quantum_junk10', {'1': '', '0': 'o', '_': ' '})
    'quantum junko'
    >>> replace_many('quantum_junk10', {'a': '0', 'b': '1'}, True)
    'quantum_junkba'
    >>> replace_many('quantum_junk10', {'': '5'})
    <TypeError traceback>

    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    if inverse:
        for k in d:
            s = s.replace(d[k], k)
    else:
        for k in d:
            s = s.replace(k, d[k])
    return s


def remove_many(s, L):
    '''
    Goes through every item of @L and removes their occurences in @s.
    
    >>> remove_many('quantum_junk10', '_0123456789')
    'quantumjunk'
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    '''
    d = {}
    for i in L:
        d[i] = ''
    return replace_many(s, d)


def keep_many(s, L):
    '''
    Goes through @s and removes all chars that are not in @L.
    
    >>> keep_many('quantum_junk10', 'abcdefghijklmnopq')
    'qanmjnk'
    
    AUTHORS:
    v0.2.0+             --> pydsigner
    v0.3.7+             --> ffao/pydsigner'''
    return ''.join(c for c in s if c in L)


def section(itr, size):
    '''
    Goes through @itr and splits it up into chunks of @size.
    
    >>> section('quantum_junk10', 3)
    ['qua', 'ntu', 'm_j', 'unk', '10']
    
    AUTHORS:
    v0.3.1+             --> pydsigner
    '''
    r = itr[:]
    res = []
    while r:
        res.append(r[:size])
        r = r[size:]
    return res


def find(itr, value, *args, **kw):
    '''
    Just like str().find(), but also works for list()'s, which have a 
    .index() like str()'s do but no .find().
    
    AUTHORS:
    v0.4.4+             --> pydsigner
    '''
    try:
        return itr.index(value, *args, **kw)
    except ValueError:
        return -1


def flatten(obj, levels=-1):
    '''
    Flattens object @obj into a list. If an iterator, @obj will be recursed 
    up @levels times if @levels is not negative, else until there are no more 
    nested lists. If the recursion limit has been reached, a list() version of 
    @obj will be returned. If @obj is not an iterable, a list containing @obj 
    will be returned.
    
    >>> flatten(1)
    [1]
    >>> flatten([1, [2, [[[3, [[[4]]]]], 5, [6]]]]]])
    [1, 2, 3, 4, 5, 6]
    >>> flatten([1, [2, [3]], 4], 1)
    [1, 2, [3], 4]
    >>> l2 = l = [0]
    >>> l.append(l2)    # a recursive object!
    >>> flatten(l, 4)   # using recursion limits to avoid errors
    [0, 0, 0, 0, 0, [0, [...]]]
    
    AUTHORS:
    v0.4.8+             --> pydsigner
    '''
    if hasattr(obj, '__iter__'):
        if levels == 0:
            L = list(obj)
        else:
            lvl = levels - 1 if levels > 0 else levels
            L = sum((flatten(sl, lvl) for sl in obj), [])
    else:
        L = [obj]
    return L
